#!/usr/bin/env python

import os
import Gnuplot
import time
import stats

gnuplot_path = "/opt/gnuplot-4.2/bin/"
os.environ['PATH'] = gnuplot_path + ":" + os.environ['PATH']

verbose = False
dbench_verbose = False

#colors = ["#0b055e", "#5f55e0", "#afaaf6", "#ebeaf8", "#f124ff", "#aaaaaa"]
colors = ["#0f067c", "#5952b8", "#9a95de", "#c2bef1", "#e4e3f9", "#ffffff"]

def min_max_normalize(l):
	minimum = min(l)
	diff_min_max = max(l)-minimum

	return [(float(i)-minimum)/(diff_min_max) for i in l]

def get_results(basedir):

	results = []
	dirs = [basedir + d for d in os.listdir(basedir) if os.path.isdir(basedir + d)]

	for d in dirs:
		if d[-4:] != ".svn":
			res_files = [ os.path.join(d,f) for f in os.listdir(d) if ".raw" in f]
			results.append(res_files)
	return results


def parse_webspec(metric):

	if verbose:
		print "*** %s ***" % (metric)
	basedir = os.getcwd() + "/webspec/"
	files = get_results(basedir)
	
	results = {16: {}, 50: {}, 84:{}}
	for res in files:
		tmp = res[0].split("_")
		sys = tmp[1]
		mode = tmp[2]
		bench = tmp[3]
		connections = int(tmp[4][:tmp[4].find("/")])

		#if verbose:
			#print "%s on %s (%s) with %i gave:" % (bench, sys, mode, connections),
		
		f = open(res[0], "r")
		vals = [float(l.split(" ")[3].strip()) for l in f if metric in l]

                #calc median

		med = vals[len(vals)/2]
		tmp = results[connections]
		tmp[sys + "_" + mode] = med

	return results
		
def do_single_file(splitter, entry, dir, metric, desc="", encoder=float, select=""):

	if verbose:
		print "\t\t\t\t*** %s ***" % (metric)
	basedir = os.getcwd() + "/" + dir + "/"
	files = os.listdir(basedir)
	files.sort()

	results = []
	for fname in files:
		vals = []
		if os.path.isfile(basedir + fname):

			tmp = fname.split("_")
			sys = tmp[1]
			if select != "":
				if select != sys:
					continue

			mode = tmp[2]
			bench = "".join(tmp[3:])

			if verbose:
				print "%s on %s (%s) %s gave:" % (bench, sys, mode, desc),
		
			f = open(basedir + "/" + fname, "r")
			for l in f:
				if metric in l:
					tmp = l.split(splitter)
					s = [x for x in tmp if x != ""]
					v = s[entry].strip()
					vals.append(encoder(v))

			vals.sort()
			med = vals[len(vals)/2]

			results.append([bench, sys, mode, med])
			if verbose:
				print med

	if verbose:
		print 
	return results

def parse_scimark(metric, arch=""):
	return do_single_file(": ", 1, "scimark2", metric, select=arch)

def parse_scimark_large(metric, arch=""):
	return do_single_file(": ", 1, "scimark2_large", metric, select=arch)

def parse_nbench(metric):
	return do_single_file(":", 1, "nbench", metric)

def parse_bashmark(metric):
	return do_single_file(":", 2, "bashmark", metric)

def parse_aim9(metric):
	return do_single_file(" ", 1, "aim9", metric)

def format_str(str):
	str = str.lower()
	tmp = str.split(" ")
	corrected = [s.capitalize() for s in tmp]
	return ' '.join(corrected)

def results_to_file(filename, metrics, parser_func, correct=1):

	f = open(filename, "w")

	f.write("\\begin{tabular}{l|r|r|r}\n")
	f.write(" \\textbf{Test} & \\textbf{Native} & \\textbf{HVM} & \\textbf{Loss}\\\\\n")
	f.write("\\hline\n")

	for m in metrics:
		res = parser_func(m)
		print res
		m = m.replace("_", "\_")
		if correct:
			m = format_str(m)
		f.write("%s & %i & %i & %.2f" % (m, res[1][3], res[0][3], 100-(res[0][3] * 100.0) / res[1][3]))
		f.write(r"\%\\" + "\n")

	f.write("\\end{tabular}\n")
	f.flush()

def generate_scimark_file(arch, filename):
	metrics = ["Composite", "FFT", "SOR", "Monte Carlo", "Sparse", "LU"]
	results_to_file(filename, metrics, parse_scimark)

def generate_scimark_large_file(arch, filename):
	metrics = ["Composite", "FFT", "SOR", "Monte Carlo", "Sparse", "LU"]
	results_to_file(filename, metrics, parse_scimark_large)

def generate_bashmark(filename):

	metrics = ["8bit integer", "16bit integer", "32bit integer", "64bit integer",
		   "32bit fp", "64bit fp", "96bit(80real) fp",
		   "128B Mem r/w (cached)", "256B Mem r/w (cached)",
		   "32Byte Mem de-/alloc", "64Byte Mem de-/alloc", "128Byte Mem de-/alloc",
		   "MThreading u-/lock"]

	results_to_file(filename, metrics, parse_bashmark)
	
def generate_nbench(filename):
	metrics = ["NUMERIC SORT", "STRING SORT", "BITFIELD",
		   "FP EMULATION", "FOURIER", "ASSIGNMENT",
		   "IDEA", "HUFFMAN", "NEURAL NET", "LU DECOMPOSITION"]

	results_to_file(filename, metrics, parse_nbench)

def generate_aim9(filename):

	tmp = filename.split(".")
	prefix = tmp[0]
	postfix = tmp[1]
	
	arit_metrics = ["add_short", "add_int", "add_long", "add_float", "add_double",
			"mul_short", "mul_int", "mul_long", "mul_float", "mul_double",
			"div_short", "div_int", "div_long", "div_float", "div_double"]

	disk_metrics = ["disk_rr", "disk_rw", "disk_rd", "disk_wrt",
			"disk_cp", "sync_disk_rw", "sync_disk_wrt",
			"sync_disk_cp", "disk_src", "link_test",
			"creat-clo", "dir_rtns_1"]

	mem_metrics = ["page_test", "brk_test", "exec_test", "fork_test",
		       "shell_rtns_1", "shell_rtns_2", "shell_rtns_3"]

	fun_call_metrics = ["fun_cal", "fun_cal1", "fun_cal2", "fun_cal15"]

	ipc_metrics = ["tcp_test", "udp_test", "fifo_test", "stream_pipe", "dgram_pipe", "pipe_cpy", "shared_memory"]

	library_metrics = ["jmp_test", "ram_copy", "num_rtns_1", "trig_rtns",
			   "mem_rtns_1", "string_rtns", "matrix_rtns", "array_rtns", 
			   "sort_rtns_1", "signal_test", "misc_rtns_1"]

	others = [ "sieve", "series_1"]

	names = ["arithmetic", "disk", "memory", "fun_call", "ipc", "library", "other"]
	all = [arit_metrics, disk_metrics, mem_metrics, fun_call_metrics, ipc_metrics, library_metrics, others]
	count = -1
	for m in all:
		count += 1
		results_to_file("".join([prefix, "_", names[count], ".", postfix]), m, parse_aim9, correct=0)


	### to do them all: (some are not present in the file, because they failed during the test)
# 	 metrics = ["add_double", "add_float", "add_long", "add_int",
# 		    "add_short", "creat-clo", "page_test", "brk_test",
# 		    "jmp_test", "signal_test", "exec_test", "fork_test",
# 		    "link_test", "disk_rr", "disk_rw", "disk_rd", "disk_wrt",
# 		    "disk_cp", "sync_disk_rw", "sync_disk_wrt", "sync_disk_cp",
# 		    "disk_src", "div_double", "div_float", "div_long",
# 		    "div_int", "div_short", "fun_cal", "fun_cal1", "fun_cal2",
# 		    "fun_cal15", "sieve", "mul_double", "mul_float", "mul_long",
# 		    "mul_int", "mul_short", "num_rtns_1", "new_raph", "trig_rtns",
# 		    "matrix_rtns", "array_rtns", "string_rtns", "mem_rtns_1",
# 		    "mem_rtns_2", "sort_rtns_1", "misc_rtns_1", "dir_rtns_1",
# 		    "shell_rtns_1", "shell_rtns_2", "shell_rtns_3", "series_1",
# 		    "shared_memory", "tcp_test", "udp_test", "fifo_test",
# 		    "stream_pipe", "dgram_pipe", "pipe_cpy", "ram_copy"]

#	 results_to_file(filename, metrics, parse_aim9, correct=0)

def parse_dbench():
	if dbench_verbose:
		print "\t\t\t\t*** dbench ***"
	basedir = os.getcwd() + "/dbench/"
	files = os.listdir(basedir)


	results = []
	for fname in files:
		cli_vals = []
		vals = []
		table = {4: [],8: [],16: [],32: [],64: [],128: [],256: []}
		if fname[-4:] != ".svn":
			tmp = fname.split("_")
			sys = tmp[1]
			mode = tmp[2]
			bench = "".join(tmp[3:])

			if dbench_verbose:
				print "%s on %s (%s) gave:" % (bench, sys, mode),
		
			f = open(basedir + "/" + fname, "r")
			for l in f:
				if "Throughput" in l:
					tmp = l.split(" ")
					val = float(tmp[1])
					clients = int(tmp[3])
					table[clients].append(val)

			for e in table.keys():
				tmp = table[e]
				tmp.sort()
				table[e] = tmp[len(tmp)/2]
				if dbench_verbose:
					print table[e]

			cli_vals = table.keys()
			vals = [table[e] for e in cli_vals]

			results.append([sys, mode, bench, cli_vals, vals])
	return results

def dbench_prepare_plotter(h, filename):

        h('set terminal pdf dashed dashlength 1.5 linewidth 1.0')
        h('set output "' + filename + '"')

	h("set style fill solid 1.200000 border -1")
	h('set grid ytics')
	h('set nokey')

	h.xlabel("Nunber of clients")
	h.ylabel("Throughput (MB/sec)")	

	h('set boxwidth 0.2')

	return h

def plot_dbench(filename):

	Gnuplot.GnuplotOpts.default_term = "pdf"
	h = Gnuplot.Gnuplot(debug=0)	
	h = dbench_prepare_plotter(h, filename)
    
	results = parse_dbench()
 	
	res = results[0]
	d1 = Gnuplot.Data(res[3], res[4], with='boxes lc rgb "' + colors[0] + '"', title=res[1] + " " + res[0])

	res = results[1]
	d2 = Gnuplot.Data(res[3], res[4], with='boxes lc rgb "' + colors[1] + '"', title=res[1] + " " + res[0])

	h('set logscale x')
	h('set xtics (4,8,16,32,64,128,256)')
	h('set xrange [2.6:384]')
	h.plot(d2, d1)

	#also make a normalized version
	normalized = []
	for i in range(len(results[0][4])):
		normalized.append((results[0][4][i] * 100.0) / results[1][4][i])

	tmp = filename.split(".")
	newfname = tmp[0] + "_normalized." + tmp[1]

	g = Gnuplot.Gnuplot(debug=0)	
	g = dbench_prepare_plotter(g, newfname)

	g('set logscale x')
	g('set xtics (4,8,16,32,64,128,256)')
	g('set xrange [2.6:384]')

	g.ylabel("Normalized throughput relative to native score")
	g('set yrange [0:100]')

	res = results[0]
	d1 = Gnuplot.Data(res[3], normalized, with='boxes lc rgb "' + colors[1] + '"', title=res[1] + " " + res[0])
	g.plot(d1)

	print "*** Plotted dbench graphs ***"

def do_aim():
	print "aim9"

def do_kernel():
	print "kernel"

def plot_netio(filename):
	metrics = ["1k bytes", "2k bytes", "4k bytes", "8k bytes", "16k bytes", "32k bytes"]

	Gnuplot.GnuplotOpts.default_term = "pdf"
	h = Gnuplot.Gnuplot(debug=0)
	h = dbench_prepare_plotter(h, filename)

	h('set boxwidth 0.15')
	h('set noxtics')
	
	#h('set key outside horiz center top')
	h('set key box')
	h('set grid')

	h('set xlabel 0,-2')

	h.xlabel("Package size (kbytes)")
	h.ylabel("Normalized throughput relative to native score")	

	bytes = []
	norm_scores = []

	count = 1

	tt_bytes = []
	tr_bytes = []
	ut_bytes = []
	ur_bytes = []

	tt_scores = []
	tr_scores = []
	ut_scores = []
	ur_scores = []

	#set up formatting...
	for metric in metrics:
		for x in range(-1,3):
			bytes.append(count+0.2*x)

		tt_bytes.append(count+(0.2*-1))
		tr_bytes.append(count+0.2*0)
		ut_bytes.append(count+(0.2*1))
		ur_bytes.append(count+(0.2*2))
		
		count += 1

	for metric in metrics:

		results = do_single_file(" ", 4, "netiotcp", metric, "Tx")
		score = (results[0][3]*100.0)/results[1][3]
		norm_scores.append(score)
		tt_scores.append(score)
		
		results = do_single_file(" ", 7, "netiotcp", metric, "Rx")
		score = (results[0][3]*100.0)/results[1][3]
		norm_scores.append(score)
		tr_scores.append(score)
		
		results = do_single_file(" ", 4, "netioudp", metric, "Tx")
		score = (results[0][3]*100.0)/results[1][3]
		norm_scores.append(score)
		ut_scores.append(score)
		
		results = do_single_file(" ", 8, "netioudp", metric, "Rx")
		score = (results[0][3]*100.0)/results[1][3]
		norm_scores.append(score)
		ur_scores.append(score)

	max_val = max(norm_scores)

	h('set xrange [' + str(min(bytes)-0.3) + ':' + str(max(bytes)+0.3) + ']')
	h("set yrange [0:" + str(max_val+23) + "]")

	d1 = Gnuplot.Data(tt_bytes, tt_scores, with='boxes lc rgb "' + colors[0] + '"', title='TCP Transmit')
	d2 = Gnuplot.Data(tr_bytes, tr_scores, with='boxes lc rgb "' + colors[1] + '"', title='TCP Receive')
	d3 = Gnuplot.Data(ut_bytes, ut_scores, with='boxes lc rgb "' + colors[2] + '"', title='UDP Transmit')
	d4 = Gnuplot.Data(ur_bytes, ur_scores, with='boxes lc rgb "' + colors[3] + '"', title='UDP Receive')

	count = -1
	for x in tr_bytes:
		count += 1
		h('set label "' + metrics[count].split(" ")[0] + '" at ' + str(x+0.0) + ',-3.5')


	h.plot(d1, d2, d3, d4)
	time.sleep(0.1)

	print "plotted netio"


def plot_sandra(filename):

	indexes = [3,3,3,6,6,5]

	metrics = ["Combined Index", "Dhrystone ALU", "Whetstone iSSE3",
		   "RAM Bandwidth Int Buff'd iSSE2", "RAM Bandwidth Float Buff'd iSSE2",
		   "Memory Latency (Random Access)"]

	count = -1
	for m in metrics:
		count += 1
		res = do_single_file(" ", indexes[count], "sandra", m)
		print "%s: %.2f" % (m, res[0][3] * 100.0 / res[1][3]),
		print "%"


# 	print do_single_file(" ", 3, "sandra", "Combined Index") #cache and memory
	
# 	print do_single_file(" ", 3, "sandra", "Dhrystone ALU") #cpu integer
# 	print do_single_file(" ", 3, "sandra", "Whetstone iSSE3") #cpu float

# 	print do_single_file(" ", 6, "sandra", "RAM Bandwidth Int Buff'd iSSE2") #memory bandwidth
# 	print do_single_file(" ", 6, "sandra", "RAM Bandwidth Float Buff'd iSSE2") #memory bandwidth

# 	print do_single_file(" ", 5, "sandra", "Memory Latency (Random Access)") #memory

def plot_webspec(filename):

	Gnuplot.GnuplotOpts.default_term = "png"
	h = Gnuplot.Gnuplot(debug=0)

        h('set terminal pdf dashed dashlength 1.5 linewidth 1.0')
        h('set output "' + filename + '"')
	h("set style fill solid 1.200000 border -1")

	h.xlabel("Nr. Connections")
	h.ylabel("Score relative to native performance")

	h('set boxwidth 0.2')

	#metrics = ["throughput", "response","min_bitrate", "max_bitrate", "ops_per_thread"]
	metrics = ["throughput"]

	table = {"d": [], "w": []}
	
	for metric in metrics:
		results = parse_webspec(metric)		
		for r in results:
			val = results[r]
			archs = val.keys()
			archs.sort()

			for i in range(len(archs)):
				if (i % 2) == 1:
					continue
				else:
					#print metric + "(" +str(r) +") " + archs[i] + "'s score relative to: " + archs[i+1], 
					#print (val[archs[i]] * 100.0) / val[archs[i+1]]
					#print val

					table[archs[i][0]].append((val[archs[i]] * 100.0) / val[archs[i+1]])

		print table
		print results.keys()

def parse_osdb(metric, arch=""):
	res = do_single_file(" ", 3, "osdb", metric, select=arch)
	return res
	
def plot_osdb(filename):
	metrics = ["Mixed IR (tup/sec)", "Mixed OLTP (tup/sec)"]
	for m in metrics:
		print parse_osdb(m)

def split_time(time):
	tmp = time.split("m")
	m = tmp[0]
	s = tmp[1].split("s")[0]
	return float(m + "." + str(float(s) / 60 *100).replace(".", ""))

def parse_kernel(metric):
	return do_single_file(" ", 1, "kernel", metric, encoder=split_time)
	

def plot_kernel(filename):

	metrics = ["real"]
	
	for metric in metrics:
		print parse_kernel(metric)
			

def parse_mmotion(arch=""):
	return do_single_file(" ", 2, "memorymotion", "time", select=arch)

def plot_arch_overall(arch, filename):

	Gnuplot.GnuplotOpts.default_term = "pdf"
	h = Gnuplot.Gnuplot(debug=0)

        h('set terminal pdf dashed dashlength 1.5 linewidth 1.0')
        h('set output "' + filename + '"')
	h("set style fill solid 1.200000 border -1")

	h.xlabel("Benchmark parameters")
	h.ylabel("Score relative to native performance")

	h('set boxwidth 0.15')
	h('set noxtics')

	h('set grid')
	
	#h('set key outside horiz center top')
	h('set key box')

	h('set xlabel 0,-2')




	#get scimark
	norm_scimark = []
	metrics = ["Composite", "FFT", "SOR", "Monte Carlo", "Sparse", "LU"]
	for m in metrics:
		res = parse_scimark(m, arch)
		norm_scimark.append(res[0][3]*100.0/res[1][3])

	#calc joint score
	scimark = stats.ldescribe(norm_scimark)[2]
	print "scimark std. dev.: " + str(stats.ldescribe(norm_scimark)[3])

	#get scimark_large
	norm_scimark = []
	metrics = ["Composite", "FFT", "SOR", "Monte Carlo", "Sparse", "LU"]
	for m in metrics:
		res = parse_scimark_large(m, arch)
		norm_scimark.append(res[0][3]*100.0/res[1][3])

	#calc joint scores
	scimark_large = stats.ldescribe(norm_scimark)[2]
	print "scimark large std. dev.: " + str(stats.ldescribe(norm_scimark)[3])
	scimark_pos = [0.6, 0.8]

	d1 = Gnuplot.Data(scimark_pos[:1], [scimark], with='boxes lc rgb "' + colors[0] + '"', title='Scimark')
	d2 = Gnuplot.Data(scimark_pos[1:], [scimark_large], with='boxes lc rgb "' + colors[1] + '"', title='Scimark Large')

	poses = scimark_pos
	vals = [scimark, scimark_large]

	#get webspec
	metrics = ["throughput"]
	results = []

	webspec_labels = []
	webspec = []
	
	for m in metrics:
		res = parse_webspec(m)
		results.append(res)
		webspec_labels.append(res.keys())
		for k in res.keys():
			print res[k]
			count = 0
			keys = res[k].keys()
			keys.sort()
			for key in keys:
				count += 1
				if arch in key:
					if "hvm" in key:
						hvm = res[k][key]
					else:
						webspec.append(hvm * 100.0 / res[k][key]) ###XXX clean this mess up!!!
	webspec_pos = [poses[-1]+0.4, poses[-1]+0.6, poses[-1]+0.8]

	poses += webspec_pos
	vals += webspec

	d3 = Gnuplot.Data(webspec_pos, webspec, with='boxes lc rgb "' + colors[2] + '"', title='SPECweb99')
		
	#get osdb
	metrics = ["Mixed IR (tup/sec)", "Mixed OLTP (tup/sec)"]
	osdb = []
	for m in metrics:
		res = parse_osdb(m, arch)
		print res
		for i in range(len(res)):
			if i:
				osdb.append(res[i-1][3] * 100.0 / res[i][3])


	osdb_labels = [m.split(" ")[1] for m in metrics]
	osdb_pos = [poses[-1]+0.4, poses[-1]+0.6]
	d4 = Gnuplot.Data(osdb_pos, osdb, with='boxes lc rgb "' + colors[3] + '"', title='OSDB')

	poses += osdb_pos 
	vals += osdb




	if arch == "debian":
               #get kernel
	       res = parse_kernel("real")
	       kernel = [res[1][3] * 100.0 / res[0][3]]
	       
	       kernel_pos = [poses[-1]+0.4]
	       d5 = Gnuplot.Data(kernel_pos, kernel, with='boxes lc rgb "' + colors[4] + '"', title='Kernel Compile')

	       poses += kernel_pos
	       vals += kernel

	#get motion
	res = parse_mmotion(arch)

	mmotion_pos = [poses[-1]+0.4]
	mmotion = [res[1][3] * 100.0 / res[0][3]]

	poses += mmotion_pos
	vals += mmotion

	d6 = Gnuplot.Data(mmotion_pos, mmotion, with='boxes lc rgb "' + colors[5] + '"', title='Memory Motion')

	scale = 1.0
	if arch == "winxp":
		scale = 3.0

	offset = str(-3.5 * scale)

	h('set xrange [' + str(min(poses)-0.3)+ ':' + str(max(poses)+0.3)+ ']')
	h('set yrange [0:' + str(max(vals)+scale * 11)+ ']')

	for i in range(len(webspec)):
		h('set label "' + str(webspec_labels[0][i]) + '" at ' + str(webspec_pos[i]-0.02) + ',' + offset)

	h('set label "' + str(osdb_labels[0]) + '" at ' + str(osdb_pos[0]-0.02) + ',' + offset)
	h('set label "' + str(osdb_labels[1]) + '" at ' + str(osdb_pos[1]-0.05) + ',' + offset)

	h('set label "' + "L" + '" at ' + str(scimark_pos[1]-0.02) + ',' + offset)

	count = -1
	for v in vals:
		count += 1
		v_str = "%.2f" % (v)
		h('set label "' + v_str + '" at ' + str(poses[count] - 0.07) + ',' + str(v + scale * 2))

	if arch == "debian":
		h.plot(d1, d2, d3, d4, d5, d6)
	else:
		h.plot(d1, d2, d3, d4, d6)

	time.sleep(0.1)

plot_arch_overall("winxp", "winxp_overall.pdf")
plot_arch_overall("debian", "debian_overall.pdf")

#generate_bashmark("tab_bashmark.tex")
#generate_nbench("tab_nbench.tex")
#generate_aim9("tab_aim9.tex")

#plot_dbench("dbench.pdf")
#plot_netio("netio.pdf")

#plot_sandra("sandra.pdf")


#XXX plot sandra
