#!/usr/bin/env python

import os,sys
import Gnuplot

VM_no_pages = (2**17) *1024 / 4096 #128 MB
total_pages = 4.0 * VM_no_pages

def sort_files(files):

    result = []

    for entry in files:
        if "xenperf" not in entry:
            continue

        result.append("")

    result.append("")
    result.append("")
    result.append("")

    for file in files:
        if "xenperf" not in file:
            continue

        parts = file.split("_")
        nr = parts[2]
        result[int(nr)] = file

    result = [f for f in files if "xenperf" in f]

    return result

def line_get_number(line):
    line = line.strip()
    while "  " in line:
        line = line.replace("  ", " ")

    parts = line.split(" ")

    return int(parts[-2])

def reset_count():
    reclaim_total = 0
    reclaim = 0
    cow_break = 0

def get_para_total_list(bdir):

    files = os.listdir(bdir)
    files = sort_files(files)

    reclaim_total_list = []

    for file in files:
        result_set = open(bdir + "/" + file, "r")
        reset_count()

        for l in result_set:
            if "commited shares" in l:
                comitted_shares = line_get_number(l)
            elif "paravirt cow break new page" in l:
                cow_break = line_get_number(l)
        reclaim_total_list.append(comitted_shares - cow_break)

    return reclaim_total_list

def get_shadow_total_list(bdir):

    files = os.listdir(bdir)
    files = sort_files(files)

    reclaim_total_list = []

    for file in files:
        result_set = open(bdir + "/" + file, "r")
        reset_count()

        #commited_shares = 0
        #cow_break = 0

        for l in result_set:
            if "reclaimed total" in l:
                comitted_shares = line_get_number(l)
        reclaim_total_list.append(comitted_shares)

    return reclaim_total_list

def plot(filename, extension):

    l1 = get_para_total_list("kernel_para")
    l2 = get_shadow_total_list("kernel_shadow")    
   
    Gnuplot.GnuplotOpts.default_term = "pdf"
    h = Gnuplot.Gnuplot(debug=0)

    if extension == "pdf":
        h('set terminal pdf dashed dashlength 1.5 ')
        #h('set terminal pdf linewidth 2.0')
        h('set output "' + filename + '.pdf"')
    elif extension == "ps":
        h("set term postscript enhanced")
        h('set output "' + filename + '.ps"')
    else:
        print "error on extension"
        sys.exit()
    
    h.xlabel("Time (Minutes)")
    h.ylabel("% VM Memory")
    
    #h('set key outside horiz center top')
    h('set key box')
    h('set grid')
    
    h("set xrange [0:50]")
    
    pl1 = []
    
    for l in l1:
        pl1.append(l * 100/ total_pages)

    pl2 = []
    
    for l in l2:
        pl2.append(l * 100/ total_pages)
        
    colors = ["#0f067c", "#5952b8", "#9a95de", "#c2bef1", "#e4e3f9", "#ffffff"]
    pd1 = Gnuplot.Data(range(1,len(pl1)+1), pl1, with='lines lw 2 lc rgb "' + colors[0] + '"', title="Paravirtualized")
    pd2 = Gnuplot.Data(range(1,len(pl2)+1), pl2, with='lines lw 2 lc rgb "' + colors[1] + '"', title="Shadow Page Tables")
    
    h.plot(pd1, pd2)

    print "Plotted: " + filename + "." + extension

plot("percentage_bw", "ps")
plot("percentage_color", "pdf")
